import React, { createContext, useContext, ReactNode } from 'react';

type APIContextType = {
  saveUpdate:(p:{ gameId:string, data:string }) => Promise<void>;
};

const APIContext = createContext<APIContextType | undefined>(undefined);

export const APIProvider: React.FC<{ children: ReactNode }> = ({ children }) => {
  const apiSend = async <T,>(p:{
    path:string;
    method:'POST'|'GET'|'PUT'|'DELETE';
    body?:any;
    query?:Record<string, string>;
  }) => {
    const query = p.query ? '?' + Object.entries(p.query).map(([k, v]) => {
      return `${encodeURIComponent(k)}=${encodeURIComponent(v)}`;
    }).join('&') : '';

    const res = await fetch(`/api/v1/${p.path}${query}`, {
      method: p.method,
      body: p.body ? JSON.stringify(p.body) : undefined,
      headers: {
        'Content-Type': 'application/json',
      },
    });

    if(!res.ok) {
      throw new Error(`API Error: ${res.status}`);
    }

    return res.json();
  }

  // Methods
  const saveUpdate:APIContextType['saveUpdate'] = async p => {
    return await apiSend({
      path: 'save',
      method: 'PUT',
      body: p
    });
  };

  // Provider
  return (
    <APIContext.Provider value={{
      saveUpdate,
    }}>
      {children}
    </APIContext.Provider>
  );
};

export const useAPI = (): APIContextType => {
  const context = useContext(APIContext);
  if (!context) {
    throw new Error('useAPI must be used within an APIProvider');
  }
  return context;
};
